@@ -33,7 +33,7 @@ import time
 from cli.ui_utils import menu_options, app_header, format_msg
 from cli.utils import (get_correct_answer, path_fixer, SetupConfig,
                        _directory_fixer, install_client_package,
-                       write_client_config)
+                       write_client_config, dev_machine_check)
 
 # framework stuff
 from not_so_tuf import (NotSoTuf, Client, FileCrypt, KeyHandler,
@@ -50,11 +50,6 @@ h = logging.NullHandler()
 h.setLevel(logging.DEBUG)
 log.addHandler(h)
 
-c = logging.StreamHandler()
-c.addFilter(logging.Filter('not_so_tuf.client'))
-c.setLevel(logging.INFO)
-log.addHandler(c)
-
 
 class Ui(object):
 
@@ -148,7 +143,7 @@ class Ui(object):
                    'enter to return to main menu and please this time update '
                    'your settings.')
             print(format_msg(msg))
-            raw_input()
+            input_()
             self.main()
 
     def scp_upload(self):
@@ -163,7 +158,7 @@ class Ui(object):
                    'enter to return to main menu and please this time update '
                    'your settings.')
             print(format_msg(msg))
-            raw_input()
+            input_()
             self.main()
 
     def keys(self):
@@ -180,14 +175,14 @@ class Ui(object):
     def show_public_key(self):
         log.debug('Show public key')
         self.key_handler.print_public_key()
-        raw_input(format_msg('Press enter to continue'))
+        input_(format_msg('Press enter to continue'))
         self.keys()
 
     def private_key_copy(self):
         log.debug('Copied private key')
         self.key_handler.copy_decrypted_private_key(self.file_crypt)
         msg = 'Private key decrypted. Press enter to continue'
-        raw_input(format_msg(msg))
+        input_(format_msg(msg))
         self.keys()
 
     def replace_keys(self):
@@ -476,10 +471,15 @@ def main():
             nst = NotSoTuf(__name__, ClientConfig())
             client = Client(nst)
             print('Current Version {}'.format(__version__))
-            print('\nChecking for updates...')
-            update = client.update_check('Not So TUF', __version__)
-            if update and not client.up_to_date:
-                client.install_restart()
+            update_check = client.update_check('Not So TUF', __version__)
+
+            # Downloading Update
+            if update_check is True:
+                downloaded = client.download()
+                # Installing Update
+                if downloaded is True and client.up_to_date is False:
+                    client.install_restart()
+
             install_client_package()
         try:
             ui = Ui()
