@@ -29,6 +29,7 @@ import sys
 from zipfile import ZipFile
 
 from not_so_tuf.compat import urllib2, execute_permission
+from not_so_tuf.config import Config
 from not_so_tuf.downloader import FileDownloader
 from not_so_tuf.patcher import Patcher
 from not_so_tuf.utils import (ChDir, rsa_verify, _version_string_to_tuple,
@@ -45,27 +46,39 @@ class Client(object):
         obj (instance): config object
     """
 
-    def __init__(self, obj=None):
+    def __init__(self, obj=None, test=False):
         if obj:
-            self.init_app(obj)
+            self.init_app(obj, test)
 
-    def init_app(self, obj):
+    def init_app(self, obj, test=False):
         """Sets up client with config values from obj
 
         Args:
             obj (instance): config object
 
         """
-        self.update_url = self._fix_update_url(obj.config.get('UPDATE_URL'))
-        self.app_name = obj.config.get('APP_NAME')
-        self.data_dir = obj.config.get('APP_DATA_DIR', None)
-        self.public_key = obj.config.get('PUBLIC_KEY')
-        self.debug = obj.config.get('DEBUG', False)
+        # Updated how client is initialized.  Can still be
+        # used the old way but the new way is way more efficient
+        # Just pass in the config object and the client takes care
+        # of the rest.  No need to initialize NotSoTuf object first!
+        if hasattr(obj, 'config'):
+            config = obj.config.copy()
+        else:
+            config = Config()
+            config.from_object(obj)
+
+        self.update_url = self._fix_update_url(config.get('UPDATE_URL'))
+        self.app_name = config.get('APP_NAME')
+        self.data_dir = config.get('APP_DATA_DIR', None)
+        self.public_key = config.get('PUBLIC_KEY')
+        self.debug = config.get('DEBUG', False)
         self.version_file = 'version.json'
         self.version_url = self.update_url + self.version_file
         self.json_data = None
         self.verified = False
         self.up_to_date = False
+        self.ready_to_update = False
+        self.highest_version = None
         if self.data_dir is None:
             extra = '.'
             if sys.platform == 'win32':
@@ -74,7 +87,8 @@ class Client(object):
                                          extra + self.app_name)
         self.update_folder = os.path.join(self.data_dir, 'update')
         self.plat = platform_
-        self._setup()
+        if test is False:
+            self._setup()
 
     def update_check(self, name, version):
         """Proxy method for :meth:`_patch_update` & :meth:`_full_update`.
@@ -94,11 +108,37 @@ class Client(object):
 
                 False - Update Failed
         """
+        self._load_json()
+        self._check_version_file_sig()
+
         self.name = name
+        self.version = version
         if not self.verified:
-            log.error('Version file not verified')
             return False
         log.info('Checking for {} updates...'.format(name))
+        try:
+            self.highest_version = self._get_highest_version(name)
+        except Exception as e:
+            log.error(str(e), exc_info=True)
+            log.warning('Supplied version number not '
+                        'found -> {}'.format(version))
+            return False
+        if _version_string_to_tuple(self.highest_version) <= \
+                _version_string_to_tuple(version):
+            log.debug('{} already updated to the latest version'.format(name))
+            log.info('Already up-to-date')
+            self.up_to_date = True
+            return False
+        # Updated is available
+        log.info('Update available')
+        self.ready_to_update = True
+        return True
+
+    def download(self):
+        if self.ready_to_update is False:
+            return
+        name = self.name
+        version = self.version
         patch_success = self._patch_update(name, version)
         if patch_success:
             log.info('Successful patch update')
@@ -106,6 +146,7 @@ class Client(object):
         log.debug('Patch updated failed...')
         update_success = self._full_update(name, version)
         if update_success:
+            log.info('Successful update')
             return True
         else:
             return False
@@ -139,9 +180,8 @@ class Client(object):
         platform_name = self.name
         if sys.platform == 'win32':
             platform_name += '.exe'
-        highest_version = self._get_highest_version(self.name)
         filename = self._get_filename_with_extension(self.name,
-                                                     highest_version)
+                                                     self.highest_version)
         with ChDir(self.update_folder):
             log.info('Extracting Update')
             z_file = ZipFile(filename)
@@ -153,9 +193,8 @@ class Client(object):
             # os.system(cmd)
 
     def _remove_version_number(self):
-        highest_version = self._get_highest_version(self.name)
         filename = self._get_filename_with_extension(self.name,
-                                                     highest_version)
+                                                     self.highest_version)
         with ChDir(self.update_folder):
             self.fixed_filename = re.sub('-' + self.plat +
                                          '-[0-9]+\.[0-9]+\.[0-9]+.*', '',
@@ -213,19 +252,6 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
 
         """
         log.info('Starting patch update')
-        try:
-            version_plat = self.json_data['y_data'][name][version][self.plat]
-            filename = version_plat['filename']
-            highest_version = self._get_highest_version(name)
-        except Exception as e:
-            log.error(str(e), exc_info=True)
-            log.warning('Supplied version number not found -> {}'.format(version))
-            return False
-        if _version_string_to_tuple(highest_version) <= \
-                _version_string_to_tuple(version):
-            log.debug('{} already updated to the latest version'.format(name))
-            self.up_to_date = True
-            return True
 
         # Just checking to see if the latest update is available to patch
         # If not we'll just do a full binary download
@@ -234,7 +260,7 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
                         'form'.format(filename))
             return False
 
-        p = Patcher(name, self.json_data, version, highest_version,
+        p = Patcher(name, self.json_data, version, self.highest_version,
                     self.update_folder)
 
         # Returns True if everything went well
@@ -257,14 +283,13 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
                 False - Update Failed
         """
         # Takes resource name and current installed version number
-        highest_version = self._get_highest_version(name)
-        url = self._get_file_url(name, highest_version)
+        url = self._get_file_url(name, self.highest_version)
         filename = self._get_filename_with_extension(name,
-                                                     highest_version)
+                                                     self.highest_version)
         log.info('Starting full update')
         with ChDir(self.update_folder):
             # _hash = self.json_data['y_data'][name][version]['md5']
-            version_ = self.json_data['y_data'][name][highest_version]
+            version_ = self.json_data['y_data'][name][self.highest_version]
             _hash = version_[self.plat]['md5']
             log.info('Downloading update...')
             fd = FileDownloader(filename, url, _hash)
@@ -286,8 +311,6 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
             if not os.path.exists(d):
                 log.debug('Creating directory: {}'.format(d))
                 os.mkdir(d)
-        self._load_json()
-        self._check_version_file_sig()
 
     def _check_version_file_sig(self):
         """Verifies version file signature"""
@@ -302,7 +325,7 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
             update_data = json.dumps(self.json_data, sort_keys=True)
             result = rsa_verify(update_data, self.sig, self.public_key)
             if result:
-                log.info('Version file verified')
+                log.debug('Version file verified')
                 self.verified = True
             else:
                 log.warning('Version file sig mismatch')
